@@ -1,14 +1,12 @@
-#!/usr/bin/env python2.5
+#!/usr/bin/env python
 import sys, tempfile, os, shutil, StringIO
 import unittest
 import logging
 import warnings
 
-# It's OK to test deprecated features
-warnings.filterwarnings("ignore", category = DeprecationWarning)
-
 # Catch silly mistakes...
 os.environ['HOME'] = '/home/idontexist'
+os.environ['LANGUAGE'] = 'C'
 
 sys.path.insert(0, '..')
 from zeroinstall.injector import qdom
@@ -37,12 +35,14 @@ os.execvp = test_execvp
 
 class BaseTest(unittest.TestCase):
 	def setUp(self):
+		warnings.resetwarnings()
 		self.config_home = tempfile.mktemp()
 		self.cache_home = tempfile.mktemp()
 		self.cache_system = tempfile.mktemp()
 		self.gnupg_home = tempfile.mktemp()
 		os.environ['GNUPGHOME'] = self.gnupg_home
 		os.environ['XDG_CONFIG_HOME'] = self.config_home
+		os.environ['XDG_CONFIG_DIRS'] = ''
 		os.environ['XDG_CACHE_HOME'] = self.cache_home
 		os.environ['XDG_CACHE_DIRS'] = self.cache_system
 		reload(basedir)
@@ -64,7 +64,8 @@ class BaseTest(unittest.TestCase):
 		self.old_path = os.environ['PATH']
 		os.environ['PATH'] = dpkgdir + ':' + self.old_path
 
-		distro._host_distribution = distro.DebianDistribution(dpkgdir)
+		distro._host_distribution = distro.DebianDistribution(dpkgdir + '/status',
+								      dpkgdir + '/pkgcache.bin')
 	
 	def tearDown(self):
 		shutil.rmtree(self.config_home)
