@@ -1,11 +1,22 @@
-#!/usr/bin/env python2.5
+#!/usr/bin/env python
 from basetest import BaseTest
 import sys, tempfile, os
-import unittest
+import unittest, logging
 
 sys.path.insert(0, '..')
-from zeroinstall.zerostore import unpack, manifest, Store
+from zeroinstall.zerostore import unpack, manifest, Store, BadDigest
 from zeroinstall import SafeException, support
+from zeroinstall.support import find_in_path
+
+def skipIf(condition, reason):
+	def wrapped(underlying):
+		if condition:
+			print "Skipped %s: %s" % (underlying.func_name, reason)
+			def run(self): pass
+			return run
+		else:
+			return underlying
+	return wrapped
 
 class AbstractTestUnpack(BaseTest):
 	def setUp(self):
@@ -33,6 +44,11 @@ class AbstractTestUnpack(BaseTest):
 		unpack.unpack_archive('ftp://foo/file.tgz', file('HelloWorld.tgz'), self.tmpdir)
 		self.assert_manifest('sha1=3ce644dc725f1d21cfcf02562c76f375944b266a')
 	
+	@skipIf(not find_in_path('hdiutil'), "not running on MacOS X; no hdiutil")
+	def testDmg(self):
+		unpack.unpack_archive('ftp://foo/file.dmg', file('HelloWorld.dmg'), self.tmpdir)
+		self.assert_manifest('sha1=3ce644dc725f1d21cfcf02562c76f375944b266a')
+	
 	def testZip(self):
 		unpack.unpack_archive('ftp://foo/file.zip', file('HelloWorld.zip'), self.tmpdir)
 		self.assert_manifest('sha1=3ce644dc725f1d21cfcf02562c76f375944b266a')
@@ -41,6 +57,10 @@ class AbstractTestUnpack(BaseTest):
 		unpack.unpack_archive('ftp://foo/file.tgz', file('HelloWorld.tgz'), self.tmpdir, extract = 'HelloWorld')
 		self.assert_manifest('sha1=3ce644dc725f1d21cfcf02562c76f375944b266a')
 	
+	def testExtractZip(self):
+		unpack.unpack_archive('ftp://foo/file.zip', file('HelloWorld.zip'), self.tmpdir, extract = 'HelloWorld')
+		self.assert_manifest('sha1=3ce644dc725f1d21cfcf02562c76f375944b266a')
+
 	def testExtractIllegal(self):
 		try:
 			unpack.unpack_archive('ftp://foo/file.tgz', file('HelloWorld.tgz'), self.tmpdir, extract = 'Hello`World`')
@@ -91,6 +111,20 @@ class AbstractTestUnpack(BaseTest):
 					   file('HelloWorld.tgz'),
 					   'http://foo/foo.tgz')
 	
+	def testBad(self):
+		logging.getLogger('').setLevel(logging.ERROR)
+
+		store = Store(self.tmpdir)
+		try:
+			store.add_archive_to_cache('sha1=3ce644dc725f1d21cfcf02562c76f375944b266b',
+						   file('HelloWorld.tgz'),
+						   'http://foo/foo.tgz')
+			assert 0
+		except BadDigest:
+			pass
+
+		logging.getLogger('').setLevel(logging.INFO)
+
 	def assert_manifest(self, required):
 		alg_name = required.split('=', 1)[0]
 		manifest.fixup_permissions(self.tmpdir)
@@ -122,12 +156,9 @@ class TestUnpackGNU(AbstractTestUnpack):
 		unpack.unpack_archive('ftp://foo/file.tar.lzma', file('HelloWorld.tar.lzma'), self.tmpdir)
 		self.assert_manifest('sha1new=290eb133e146635fe37713fd58174324a16d595f')
 
-suite = unittest.TestSuite()
-if unpack._gnu_tar():
-	suite.addTest(unittest.makeSuite(TestUnpackGNU))
-else:
+if not unpack._gnu_tar():
 	print "No GNU tar: SKIPPING tests"
-suite.addTest(unittest.makeSuite(TestUnpackPython))
+	del globals()['TestUnpackGNU']
 
 if __name__ == '__main__':
-	unittest.TextTestRunner(verbosity=2).run(suite)
+	unittest.main()
