@@ -35,22 +35,12 @@ def have_source_for(policy, interface):
 	# - We have a feed of type 'src' (not fetched), or
 	# - We have a source implementation in a regular feed
 	have_src = False
-	for f in interface.feeds:
+	for f in iface_cache.get_feed_imports(interface):
 		if f.machine == 'src':
 			return True
 	# Don't have any src feeds. Do we have a source implementation
 	# as part of a regular feed?
-	impls = interface.implementations.values()
-	for f in policy.usable_feeds(interface):
-		try:
-			feed_iface = iface_cache.get_interface(f.uri)
-			if feed_iface.implementations:
-				impls.extend(feed_iface.implementations.values())
-		except zeroinstall.NeedDownload:
-			pass	# OK, will get called again later
-		except Exception, ex:
-			warn(_("Failed to load feed '%(feed)s': %(exception)s"), {'feed': f.uri, 'exception': str(ex)})
-	for x in impls:
+	for x in iface_cache.get_implementations(interface):
 		if x.machine == 'src':
 			return True
 	return False
@@ -86,7 +76,7 @@ class Description:
 		except ImportError, ValueError:
 			return time.ctime(secs)
 
-	def set_details(self, interface):
+	def set_details(self, feed):
 		buffer = self.buffer
 		heading_style = self.heading_style
 
@@ -94,22 +84,26 @@ class Description:
 
 		iter = buffer.get_start_iter()
 
-		buffer.insert_with_tags(iter,
-			'%s ' % interface.get_name(), heading_style)
-		buffer.insert(iter, '(%s)' % interface.summary)
+		if feed is None:
+			buffer.insert(iter, 'Not yet downloaded.')
+			return
 
-		buffer.insert(iter, '\n%s\n' % interface.uri)
+		buffer.insert_with_tags(iter,
+			'%s ' % feed.get_name(), heading_style)
+		buffer.insert(iter, '(%s)' % feed.summary)
+
+		buffer.insert(iter, '\n%s\n' % feed.url)
 
 		# (converts to local time)
-		if interface.last_modified:
-			buffer.insert(iter, '\n' + _('Last upstream change: %s') % self.strtime(interface.last_modified))
+		if feed.last_modified:
+			buffer.insert(iter, '\n' + _('Last upstream change: %s') % self.strtime(feed.last_modified))
 
-		if interface.last_checked:
-			buffer.insert(iter, '\n' + _('Last checked: %s') % self.strtime(interface.last_checked))
+		if feed.last_checked:
+			buffer.insert(iter, '\n' + _('Last checked: %s') % self.strtime(feed.last_checked))
 
-		last_check_attempt = iface_cache.get_last_check_attempt(interface.uri)
+		last_check_attempt = iface_cache.get_last_check_attempt(feed.url)
 		if last_check_attempt:
-			if interface.last_checked and interface.last_checked >= last_check_attempt:
+			if feed.last_checked and feed.last_checked >= last_check_attempt:
 				pass	# Don't bother reporting successful attempts
 			else:
 				buffer.insert(iter, '\n' + _('Last check attempt: %s (failed or in progress)') %
@@ -117,13 +111,13 @@ class Description:
 
 		buffer.insert_with_tags(iter, '\n\n' + _('Description') + '\n', heading_style)
 
-		paragraphs = [format_para(p) for p in (interface.description or "-").split('\n\n')]
+		paragraphs = [format_para(p) for p in (feed.description or "-").split('\n\n')]
 
 		buffer.insert(iter, '\n\n'.join(paragraphs))
 		buffer.insert(iter, '\n')
 
 		need_gap = True
-		for x in interface.get_metadata(namespaces.XMLNS_IFACE, 'homepage'):
+		for x in feed.get_metadata(namespaces.XMLNS_IFACE, 'homepage'):
 			if need_gap:
 				buffer.insert(iter, '\n')
 				need_gap = False
@@ -131,7 +125,7 @@ class Description:
 			buffer.insert_with_tags(iter, '%s\n' % x.content, self.link_style)
 
 		buffer.insert_with_tags(iter, '\n' + _('Signatures') + '\n', heading_style)
-		sigs = iface_cache.get_cached_signatures(interface.uri)
+		sigs = iface_cache.get_cached_signatures(feed.url)
 		if sigs:
 			for sig in sigs:
 				if isinstance(sig, gpg.ValidSig):
@@ -144,7 +138,7 @@ class Description:
 					buffer.insert_with_tags(iter, _('Valid signature by "%(name)s"\n- Dated: %(sig_date)s\n- Fingerprint: %(sig_fingerprint)s\n') %
 							{'name': name, 'sig_date': time.strftime('%c', time.localtime(sig.get_timestamp())), 'sig_fingerprint': sig.fingerprint})
 					if not sig.is_trusted():
-						if interface.uri.startswith('/'):
+						if feed.url.startswith('/'):
 							buffer.insert_with_tags(iter, _('WARNING: This key is not in the trusted list') + '\n')
 						else:
 							buffer.insert_with_tags(iter, _('WARNING: This key is not in the trusted list (either you removed it, or '
@@ -152,7 +146,7 @@ class Description:
 				else:
 					buffer.insert_with_tags(iter, '%s\n' % sig)
 		else:
-			buffer.insert_with_tags(iter, _('No signature information (old style interface or out-of-date cache)') + '\n')
+			buffer.insert_with_tags(iter, _('No signature information (old style feed or out-of-date cache)') + '\n')
 
 class Feeds:
 	URI = 0
@@ -181,7 +175,7 @@ class Feeds:
 		def remove_feed(button):
 			model, iter = self.tv.get_selection().get_selected()
 			feed_uri = model[iter][Feeds.URI]
-			for x in interface.feeds:
+			for x in interface.extra_feeds:
 				if x.uri == feed_uri:
 					if x.user_override:
 						interface.extra_feeds.remove(x)
@@ -208,29 +202,28 @@ class Feeds:
 	
 	def build_model(self):
 		usable_feeds = frozenset(self.policy.usable_feeds(self.interface))
-		unusable_feeds = frozenset(self.interface.feeds) - usable_feeds
+		unusable_feeds = frozenset(iface_cache.get_feed_imports(self.interface)) - usable_feeds
 
 		out = [[self.interface.uri, None, True]]
 
-		if self.interface.feeds:
-			for feed in usable_feeds:
-				out.append([feed.uri, feed.arch, True])
-			for feed in unusable_feeds:
-				out.append([feed.uri, feed.arch, False])
+		for feed in usable_feeds:
+			out.append([feed.uri, feed.arch, True])
+		for feed in unusable_feeds:
+			out.append([feed.uri, feed.arch, False])
 		return out
 
 	def sel_changed(self, sel):
 		model, miter = sel.get_selected()
 		if not miter: return	# build in progress
-		iface = model[miter][Feeds.URI]
+		feed_url = model[miter][Feeds.URI]
 		# Only enable removing user_override feeds
 		enable_remove = False
-		for x in self.interface.feeds:
-			if x.uri == iface:
+		for x in self.interface.extra_feeds:
+			if x.uri == feed_url:
 				if x.user_override:
 					enable_remove = True
 		self.remove_feed_button.set_sensitive( enable_remove )
-		self.description.set_details(iface_cache.get_interface(iface))
+		self.description.set_details(iface_cache.get_feed(feed_url))
 	
 	def updated(self):
 		new_lines = self.build_model()
@@ -332,7 +325,8 @@ class Properties:
 			# The Solver didn't get this far, but we should still display them!
 			ranked_items = [(impl, _("(solve aborted before here)"))
 					for impl in self.interface.implementations.values()]
-			ranked_items.sort()
+		# Always sort by version
+		ranked_items.sort()
 		self.use_list.set_items(ranked_items)
 
 @tasks.async
@@ -397,7 +391,7 @@ def add_remote_feed(policy, parent, interface):
 						elif interface.uri not in iface.feed_for:
 							error(_("This is not a feed for '%(uri)s'.\nOnly for:\n%(feed_for)s") %
 								{'uri': interface.uri, 'feed_for': '\n'.join(iface.feed_for)})
-						elif iface.uri in [f.uri for f in interface.feeds]:
+						elif iface.uri in [f.uri for f in interface.extra_feeds]:
 							error(_("Feed from '%s' has already been added!") % iface.uri)
 						else:
 							interface.extra_feeds.append(Feed(iface.uri, arch = None, user_override = True))
@@ -424,7 +418,7 @@ def add_local_feed(policy, interface):
 				raise Exception(_("Not a valid feed for '%(uri)s'; this is a feed for:\n%(feed_for)s") %
 						{'uri': interface.uri,
 						'feed_for': '\n'.join([f.uri for f in feed_targets])})
-			if interface.get_feed(feed):
+			if feed in [f.uri for f in interface.extra_feeds]:
 				dialog.alert(None, _('This feed is already registered.'))
 			else:
 				interface.extra_feeds.append(Feed(feed, user_override = True, arch = None))
@@ -501,12 +495,11 @@ In off-line mode, only cached implementations are considered for use.
 Arch indicates what kind of computer system the implementation is for, or 'any' \
 if it works with all types of system.""") + '\n'),
 (_('Sort order'), '\n' +
-_("""The implementations are listed in the injector's currently preferred order (the one \
-at the top will actually be used). Usable implementations all come before unusable \
-ones.
+_("""The implementations are ordered by version number (highest first), with the \
+currently selected one in bold. This is the "best" usable version.
 
 Unusable ones are those for incompatible \
-architectures, those marked as 'buggy', versions explicitly marked as incompatible with \
+architectures, those marked as 'buggy' or 'insecure', versions explicitly marked as incompatible with \
 another interface you are using and, in off-line mode, uncached implementations. Unusable \
 implementations are shown crossed out.
 
