@@ -12,11 +12,19 @@ The Python implementation of the Zero Install injector is divided into four sub-
 @var _: a function for translating strings using the zero-install domain (for use internally by Zero Install)
 """
 
-version = '0.44'
+version = '0.48'
 
 import gettext
+from os.path import dirname, join
 
-translation = gettext.translation('zero-install', fallback = True)
+try:
+	localedir = None
+	translation = gettext.translation('zero-install', fallback = False)
+except:
+	localedir = join(dirname(dirname(__file__)), 'locale')
+	translation = gettext.translation('zero-install',
+				localedir = localedir,
+				fallback = True)
 _ = translation.ugettext
 
 class SafeException(Exception):
