@@ -15,7 +15,7 @@ well-known variables.
 
 from zeroinstall import _
 import os, re
-from logging import info, debug
+from logging import info, debug, warn
 from zeroinstall import SafeException, version
 from zeroinstall.injector.namespaces import XMLNS_IFACE
 
@@ -39,7 +39,17 @@ class InvalidInterface(SafeException):
 	"""Raised when parsing an invalid feed."""
 	def __init__(self, message, ex = None):
 		if ex:
-			message += "\n\n(exact error: %s)" % ex
+			try:
+				message += "\n\n(exact error: %s)" % ex
+			except:
+				# Some Python messages have type str but contain UTF-8 sequences.
+				# (e.g. IOException). Adding these to a Unicode 'message' (e.g.
+				# after gettext translation) will cause an error.
+				import codecs
+				decoder = codecs.lookup('utf-8')
+				decex = decoder.decode(str(ex), errors = 'replace')[0]
+				message += "\n\n(exact error: %s)" % decex
+
 		SafeException.__init__(self, message)
 
 def _split_arch(arch):
@@ -57,7 +67,7 @@ def _split_arch(arch):
 def _join_arch(osys, machine):
 	if osys == machine == None: return None
 	return "%s-%s" % (osys or '*', machine or '*')
-	
+
 class Stability(object):
 	"""A stability rating. Each implementation has an upstream stability rating and,
 	optionally, a user-set rating."""
@@ -68,10 +78,10 @@ class Stability(object):
 		self.description = description
 		assert name not in stability_levels
 		stability_levels[name] = self
-	
+
 	def __cmp__(self, other):
 		return cmp(self.level, other.level)
-	
+
 	def __str__(self):
 		return self.name
 
@@ -135,7 +145,7 @@ class Restriction(object):
 	__slots__ = []
 
 	def meets_restriction(self, impl):
-		"""Called by the L{Solver} to check whether a particular implementation is acceptable.
+		"""Called by the L{solver.Solver} to check whether a particular implementation is acceptable.
 		@return: False if this implementation is not a possibility
 		@rtype: bool
 		"""
@@ -357,6 +367,20 @@ class Recipe(RetrievalMethod):
 	
 	size = property(lambda self: sum([x.size for x in self.steps]))
 
+class DistributionSource(RetrievalMethod):
+	"""A package that is installed using the distribution's tools (including PackageKit).
+	@ivar package_id: the package name, in a form recognised by the distribution's tools
+	@type package_id: str
+	@ivar size: the download size in bytes
+	@type size: int"""
+
+	__slots__ = ['package_id', 'size']
+
+	def __init__(self, package_id, size):
+		RetrievalMethod.__init__(self)
+		self.package_id = package_id
+		self.size = size
+
 class Implementation(object):
 	"""An Implementation is a package which implements an Interface.
 	@ivar download_sources: list of methods of getting this implementation
@@ -370,13 +394,19 @@ class Implementation(object):
 	@ivar user_stability: the stability as set by the user
 	@type upstream_stability: [insecure | buggy | developer | testing | stable | packaged | preferred]
 	@ivar langs: natural languages supported by this package
+	@type langs: str
 	@ivar requires: interfaces this package depends on
+	@type requires: [L{Dependency}]
 	@ivar main: the default file to execute when running as a program
 	@ivar metadata: extra metadata from the feed
 	@type metadata: {"[URI ]localName": str}
 	@ivar id: a unique identifier for this Implementation
 	@ivar version: a parsed version number
 	@ivar released: release date
+	@ivar local_path: the directory containing this local implementation, or None if it isn't local (id isn't a path)
+	@type local_path: str | None
+	@ivar requires_root_install: whether the user will need admin rights to use this
+	@type requires_root_install: bool
 	"""
 
 	# Note: user_stability shouldn't really be here
@@ -397,7 +427,7 @@ class Implementation(object):
 		self.version = None
 		self.released = None
 		self.download_sources = []
-		self.langs = None
+		self.langs = ""
 		self.machine = None
 		self.bindings = []
 
@@ -412,7 +442,13 @@ class Implementation(object):
 
 	def __cmp__(self, other):
 		"""Newer versions come first"""
-		return cmp(other.version, self.version)
+		d = cmp(other.version, self.version)
+		if d: return d
+		# If the version number is the same, just give a stable sort order, and
+		# ensure that two different implementations don't compare equal.
+		d = cmp(other.feed.url, self.feed.url)
+		if d: return d
+		return cmp(other.id, self.id)
 
 	def get_version(self):
 		"""Return the version as a string.
@@ -423,41 +459,56 @@ class Implementation(object):
 	arch = property(lambda self: _join_arch(self.os, self.machine))
 
 	os = None
+	local_path = None
+	digests = None
+	requires_root_install = False
 
 class DistributionImplementation(Implementation):
 	"""An implementation provided by the distribution. Information such as the version
 	comes from the package manager.
 	@since: 0.28"""
-	__slots__ = ['installed']
 
-	def __init__(self, feed, id):
+	def __init__(self, feed, id, distro):
 		assert id.startswith('package:')
 		Implementation.__init__(self, feed, id)
-		self.installed = True
-	
+		self.distro = distro
+
+	@property
+	def requires_root_install(self):
+		return not self.installed
+
+	@property
+	def installed(self):
+		return self.distro.get_installed(self.id)
+
 class ZeroInstallImplementation(Implementation):
 	"""An implementation where all the information comes from Zero Install.
+	@ivar digests: a list of "algorith=value" strings (since 0.45)
+	@type digests: [str]
 	@since: 0.28"""
-	__slots__ = ['os', 'size']
+	__slots__ = ['os', 'size', 'digests', 'local_path']
 
-	def __init__(self, feed, id):
+	def __init__(self, feed, id, local_path):
 		"""id can be a local path (string starting with /) or a manifest hash (eg "sha1=XXX")"""
+		assert not id.startswith('package:'), id
 		Implementation.__init__(self, feed, id)
 		self.size = None
 		self.os = None
+		self.digests = []
+		self.local_path = local_path
 
 	# Deprecated
 	dependencies = property(lambda self: dict([(x.interface, x) for x in self.requires
 						   if isinstance(x, InterfaceDependency)]))
-	
+
 	def add_download_source(self, url, size, extract, start_offset = 0, type = None):
 		"""Add a download source."""
 		self.download_sources.append(DownloadSource(self, url, size, extract, start_offset, type))
-	
+
 	def set_arch(self, arch):
 		self.os, self.machine = _split_arch(arch)
 	arch = property(lambda self: _join_arch(self.os, self.machine), set_arch)
-	
+
 class Interface(object):
 	"""An Interface represents some contract of behaviour.
 	@ivar uri: the URI for this interface.
@@ -465,7 +516,7 @@ class Interface(object):
 	Implementations at this level or higher are preferred.
 	Lower levels are used only if there is no other choice.
 	"""
-	__slots__ = ['uri', 'stability_policy', '_main_feed', 'extra_feeds']
+	__slots__ = ['uri', 'stability_policy', 'extra_feeds']
 
 	implementations = property(lambda self: self._main_feed.implementations)
 	name = property(lambda self: self._main_feed.name)
@@ -495,12 +546,13 @@ class Interface(object):
 
 	def reset(self):
 		self.extra_feeds = []
-		self._main_feed = _dummy_feed
 		self.stability_policy = None
 
 	def get_name(self):
-		if self._main_feed is not _dummy_feed:
-			return self._main_feed.get_name()
+		from zeroinstall.injector.iface_cache import iface_cache
+		feed = iface_cache.get_feed(self.uri)
+		if feed:
+			return feed.get_name()
 		return '(' + os.path.basename(self.uri) + ')'
 	
 	def __repr__(self):
@@ -511,14 +563,27 @@ class Interface(object):
 		self.stability_policy = new
 	
 	def get_feed(self, url):
+		#import warnings
+		#warnings.warn("use iface_cache.get_feed instead", DeprecationWarning, 2)
 		for x in self.extra_feeds:
 			if x.uri == url:
 				return x
-		return self._main_feed.get_feed(url)
+		#return self._main_feed.get_feed(url)
+		return None
 	
 	def get_metadata(self, uri, name):
 		return self._main_feed.get_metadata(uri, name)
 
+	@property
+	def _main_feed(self):
+		#import warnings
+		#warnings.warn("use the feed instead", DeprecationWarning, 3)
+		from zeroinstall.injector.iface_cache import iface_cache
+		feed = iface_cache.get_feed(self.uri)
+		if feed is None:
+			return _dummy_feed
+		return feed
+
 def _merge_attrs(attrs, item):
 	"""Add each attribute of item to a copy of attrs and return the copy.
 	@type attrs: {str: str}
@@ -623,7 +688,7 @@ class ZeroInstallFeed(object):
 				feed_src = x.getAttribute('src')
 				if not feed_src:
 					raise InvalidInterface(_('Missing "src" attribute in <feed>'))
-				if feed_src.startswith('http:') or local_path:
+				if feed_src.startswith('http:') or feed_src.startswith('https:') or local_path:
 					self.feeds.append(Feed(feed_src, x.getAttribute('arch'), False, langs = x.getAttribute('langs')))
 				else:
 					raise InvalidInterface(_("Invalid feed URL '%s'") % feed_src)
@@ -635,6 +700,8 @@ class ZeroInstallFeed(object):
 		if not self.summary:
 			raise InvalidInterface(_("Missing <summary> in feed"))
 
+		package_impls = [0, []]		# Best score so far and packages with that score
+
 		def process_group(group, group_attrs, base_depends, base_bindings):
 			for item in group.childNodes:
 				if item.uri != XMLNS_IFACE: continue
@@ -666,7 +733,14 @@ class ZeroInstallFeed(object):
 				elif item.name == 'implementation':
 					process_impl(item, item_attrs, depends, bindings)
 				elif item.name == 'package-implementation':
-					process_native_impl(item, item_attrs, depends)
+					distro_names = item_attrs.get('distributions', '')
+					for distro_name in distro_names.split(' '):
+						score = distro.get_score(distro_name)
+						if score > package_impls[0]:
+							package_impls[0] = score
+							package_impls[1] = []
+						if score == package_impls[0]:
+							package_impls[1].append((item, item_attrs, depends))
 				else:
 					assert 0
 
@@ -674,17 +748,22 @@ class ZeroInstallFeed(object):
 			id = item.getAttribute('id')
 			if id is None:
 				raise InvalidInterface(_("Missing 'id' attribute on %s") % item)
-			if local_dir and (id.startswith('/') or id.startswith('.')):
-				impl = self._get_impl(os.path.abspath(os.path.join(local_dir, id)))
+			local_path = item_attrs.get('local-path')
+			if local_dir and local_path:
+				abs_local_path = os.path.abspath(os.path.join(local_dir, local_path))
+				impl = ZeroInstallImplementation(self, id, abs_local_path)
+			elif local_dir and (id.startswith('/') or id.startswith('.')):
+				# For old feeds
+				id = os.path.abspath(os.path.join(local_dir, id))
+				impl = ZeroInstallImplementation(self, id, id)
 			else:
-				if '=' not in id:
-					raise InvalidInterface(_('Invalid "id"; form is "alg=value" (got "%s")') % id)
-				alg, sha1 = id.split('=')
-				try:
-					long(sha1, 16)
-				except Exception, ex:
-					raise InvalidInterface(_('Bad SHA1 attribute: %s') % ex)
-				impl = self._get_impl(id)
+				impl = ZeroInstallImplementation(self, id, None)
+				if '=' in id:
+					# In older feeds, the ID was the (single) digest
+					impl.digests.append(id)
+			if id in self.implementations:
+				warn(_("Duplicate ID '%s' in feed '%s'"), id, self)
+			self.implementations[id] = impl
 
 			impl.metadata = item_attrs
 			try:
@@ -704,7 +783,7 @@ class ZeroInstallFeed(object):
 			impl.main = item_main
 
 			impl.released = item_attrs.get('released', None)
-			impl.langs = item_attrs.get('langs', None)
+			impl.langs = item_attrs.get('langs', '')
 
 			size = item.getAttribute('size')
 			if size:
@@ -737,6 +816,10 @@ class ZeroInstallFeed(object):
 							extract = elem.getAttribute('extract'),
 							start_offset = _get_long(elem, 'start-offset'),
 							type = elem.getAttribute('type'))
+				elif elem.name == 'manifest-digest':
+					for aname, avalue in elem.attrs.iteritems():
+						if ' ' not in aname:
+							impl.digests.append('%s=%s' % (aname, avalue))
 				elif elem.name == 'recipe':
 					recipe = Recipe()
 					for recipe_step in elem.childNodes:
@@ -764,7 +847,10 @@ class ZeroInstallFeed(object):
 
 			def factory(id):
 				assert id.startswith('package:')
-				impl = self._get_impl(id)
+				if id in self.implementations:
+					warn(_("Duplicate ID '%s' for DistributionImplementation"), id)
+				impl = DistributionImplementation(self, id, distro)
+				self.implementations[id] = impl
 
 				impl.metadata = item_attrs
 
@@ -785,20 +871,29 @@ class ZeroInstallFeed(object):
 			root_attrs['main'] = main
 		process_group(feed_element, root_attrs, [], [])
 
+		for args in package_impls[1]:
+			process_native_impl(*args)
+
 	def get_name(self):
 		return self.name or '(' + os.path.basename(self.url) + ')'
 	
 	def __repr__(self):
 		return _("<Feed %s>") % self.url
 	
+	"""@deprecated"""
 	def _get_impl(self, id):
-		if id not in self.implementations:
-			if id.startswith('package:'):
-				impl = DistributionImplementation(self, id)
-			else:
-				impl = ZeroInstallImplementation(self, id)
-			self.implementations[id] = impl
-		return self.implementations[id]
+		assert id not in self.implementations
+
+		if id.startswith('.') or id.startswith('/'):
+			id = os.path.abspath(os.path.join(self.url, id))
+			local_path = id
+			impl = ZeroInstallImplementation(self, id, local_path)
+		else:
+			impl = ZeroInstallImplementation(self, id, None)
+			impl.digests.append(id)
+
+		self.implementations[id] = impl
+		return impl
 	
 	def set_stability_policy(self, new):
 		assert new is None or isinstance(new, Stability)
